import click
from bifrostx.server import start_server
from bifrostx import __version__


@click.group
@click.version_option(version=__version__, prog_name="BifrostX")
def cli():
    pass


@cli.command
def server():
    start_server()


@cli.command
def install():
    # TODO: install extnsions from github or zip
    click.echo("Install Not implemented Yet")


@cli.command
def uninstall():
    # TODO: uninstall extnsions
    click.echo("Uninstall Not implemented Yet")


if __name__ == "__main__":
    cli()



from loguru import logger
from bifrostx.config import Config
import sys

logger.remove()


logger.add(
    sys.stderr,
    level=Config.LOG_LEVEL,
    format="<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> - "
    "<level>{level}</level> - "
    "<cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> - <level>{message}</level>",
)



from abc import ABC
from bifrostx.config import Config
from pydantic import BaseModel
from bifrostx.utils.logger import logger
from typing import Type, Optional


class BaseInterface(ABC):
    instance_config_schema: Type[BaseModel] = None

    def __init__(self, instance_config):
        self.instance_config = (
            self.instance_config_schema(**instance_config)
            if self.instance_config_schema
            else None
        )

    @classmethod
    def get_instance(
        cls, instance_id: Optional[str] = None, adapter_name: Optional[str] = None
    ):
        if instance_id and ":" in instance_id and adapter_name is None:
            adapter_name, instance_id = instance_id.split(":", 1)
        try:
            if cls.__module__.startswith("Interfaces"):
                from .register import InterfaceRegister

                info = InterfaceRegister.get_interface(cls.__module__)
                if info:
                    return info.get_adapter_instance(
                        adapter_name=f"Adapters.{adapter_name}", instance_id=instance_id
                    )
                raise ValueError(f"未找到{cls.__module__}实例")
            else:
                configs = Config.get_extension_config(
                    module=f"Adapters.{adapter_name}"
                    if adapter_name
                    else cls.__module__,
                    instances=True,
                )
                if configs:
                    if not instance_id:
                        return cls(list(configs.values())[0])
                    elif instance_id and instance_id in configs:
                        return cls(configs[instance_id])
                raise ValueError(f"未找到{cls.__module__}实例")
        except ValueError as ex:
            logger.warning(ex)
            return None



from pathlib import Path
import sys
from bifrostx.interface.register import InterfaceRegister
from bifrostx.adapter.register import AdapterRegister
from bifrostx.component.register import ComponentRegister
from bifrostx.config import Config
from bifrostx.utils.logger import logger


def init_extension_dir():
    """
    初始化扩展目录
    """
    logger.info("Initializing extension directory...")
    ext_dir = Path(Config.EXTENSION_DIR)
    if str(ext_dir.absolute()) not in sys.path:
        sys.path.append(str(ext_dir.absolute()))
    init_dirs = {
        "Interfaces": InterfaceRegister,
        "Adapters": AdapterRegister,
        "Components": ComponentRegister,
    }
    for init_dir, register in init_dirs.items():
        item_dir = ext_dir.joinpath(init_dir)
        item_dir.mkdir(parents=True, exist_ok=True)
        item_dir.joinpath("__init__.py").touch(exist_ok=True)
        for item in item_dir.iterdir():
            if item.is_dir() and item.name not in ("__pycache__",):
                # 注册拓展
                register.register(item.name)



from typing import Any, Tuple, Type, Dict, Union

import tomli
from pathlib import Path

from pydantic.fields import FieldInfo
from pydantic_settings import (
    BaseSettings,
    PydanticBaseSettingsSource,
    SettingsConfigDict,
)


class TomlConfigSettingSource(PydanticBaseSettingsSource):
    def get_field_value(
        self, field: FieldInfo, field_name: str
    ) -> tuple[Any, str, bool]:
        config_file = self.config.get("bifrostx_config", "config.toml")
        file_path = Path(config_file)
        if file_path.exists():
            data = tomli.loads(file_path.read_text("utf8"))
            field_value = data.get(field_name)
            return field_value, field_name, False
        else:
            file_path.touch()
            return None, field_name, False

    def prepare_field_value(
        self, field_name: str, field: FieldInfo, value: Any, value_is_complex: bool
    ) -> Any:
        return value

    def __call__(self, *args, **kwargs):
        d = {}
        for field_name, field in self.settings_cls.model_fields.items():
            field_value, field_key, value_is_complex = self.get_field_value(
                field, field_name
            )
            field_value = self.prepare_field_value(
                field_name, field, field_value, value_is_complex
            )
            if field_value is not None:
                d[field_key] = field_value

        return d


class ConfigObject(BaseSettings):
    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: Type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> Tuple[PydanticBaseSettingsSource, ...]:
        return (init_settings, TomlConfigSettingSource(settings_cls))

    def get_extension_config(self, module: Union[str, Type], instances=False) -> Dict:
        key = "instances" if instances else "config"
        module_name = module
        if not isinstance(module, str):
            module_name = module.__name__
        modules = module_name.split(".")[:2]
        if len(modules) == 2 and hasattr(self, modules[0]):
            return getattr(self, modules[0]).get(modules[1], {}).get(key, {})
        raise ValueError(f"未找到相关{module_name}配置")

    LOG_LEVEL: str = "DEBUG"
    EXTENSION_DIR: str = "."
    FONTEND_DIR: str = "fontend"
    Adapters: Dict = {}
    Interfaces: Dict = {}
    Components: Dict = {}


Config = ConfigObject()
